#!/usr/bin/perl

#
# $Id: lipm_fastasplitter.pl 1620 2018-08-06 14:18:40Z gouzy $
#

=pod 

=head1 NAME

lipm_fastasplitter.pl

=head1 SYNOPSIS

lipm_fastasplitter.pl --in fastafile --outprefix string [--num_per_slice integer] [--maxlength_per_slice integer]

=head1 DESCRIPTION


=cut


BEGIN
{
        use File::Basename;
        my $dirprg=dirname($0);
        unshift @INC, "$dirprg/../corelib";
        unshift @INC, "$dirprg/../bin";
		$ENV{PATH} .= ":$dirprg";
		$ENV{LANG} = "C";
        $VERSION = do { my @r = (q$Rev: 1620 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }
}

use strict;
use warnings;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;

my $NUM_PER_SLICE = 1000000;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'help','in=s','outprefix=s','num_per_slice=i','maxlength_per_slice=i');
	$o_param->AssertDefined('in');
	$o_param->SetUnlessDefined('outprefix',$o_param->Get('in').".slice");
	$o_param->SetUnlessDefined('num_per_slice',$NUM_PER_SLICE);
	my $maxlen=0;
	if ( $o_param->IsDefined('maxlength_per_slice') )
	{
		$o_param->AssertInteger('maxlength_per_slice');
		$maxlen=$o_param->Get('maxlength_per_slice');
	}
	my $f_in = $o_param->GetStreamIn('in');
	my $cpt=0;
	my $slice=1;
	my $outp = $o_param->Get('outprefix');
	my $num  = $o_param->Get('num_per_slice');
	my $out = &GetOut($outp,$slice++);
	my $f_out = &GetStreamOut($out);
	my $sum=0;
	while(my $l=<$f_in>)
	{
		if ( $l =~ /^>/ )
		{
			if ( ( ++$cpt > $num ) || ( $maxlen && $sum > $maxlen ) )
			{
				$out = &GetOut($outp,$slice++);
				$f_out->close;
				$f_out = &GetStreamOut($out);
				$cpt=1;
				$sum=0;
			}
		}
		else
		{
			$sum+=(length($l)-1);
		}
		print $f_out $l;
	}
	$f_in->close;
	$f_out->close;
}

sub GetOut
{
my($outp,$slice)=@_;

	return sprintf("$outp%05d",$slice);
}

sub Usage
{
print STDERR<<END
$0
	--help
Mandatory
	--in            fastafilename         compressed format allowed
Optional
	--outprefix      string               output file prefix 
	--num_per_slice  integer              [$NUM_PER_SLICE] number of sequence per slice
	--maxlength_per_slice  integer        [0] maximun number of nucleotides
END
}
